'''
Christian Pluchar- LIGO SURF17

Photon Absorption Analysis
Uses code from calibrationAnalysis for I/O operations

Takes a group of images, and pixel by pixel, finds the total count of each pixel in the
matrix that comprises the image. It then takes the same pixel in the group of images and
finds the mean and standard deviation. 

Rawpy turns each RAW picture into an matrix. Then add each matrix into an array (3d array now)
and iterate through the depth keeping row and column constant (depth, row, column). Add up


ALL IMAGES MUST BE THE SAME SIZE

Depenencies:
Rawpy (for opening Raw images and extracting data from them)
imageoio (Rawpy uses this)
Numpy (for some simple data processing)
Glob (included, for opening groups of images)
Scipy
matplotlib

'''

import rawpy
import imageio
import numpy as np
from PIL import Image
import glob
import os
import csv

import numpy as np
from makePlots import *

import matplotlib

matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from scipy.optimize import curve_fit

directory = 'data_7_13//laser//temp//'
largeAnalysis = 100

#allImages is a 1D list that has all of the raw images in it
allImages = []
os.chdir(directory)


#gets all the images in the directory and adds them to allImages
#NOTE: will have to change the directory back the location of analysis.py
for image in glob.glob('*.NEF'):
	allImages.append(image)
	print(image)

os.chdir('..')
os.chdir('..')
os.chdir('..')

print(allImages)

#gets the number of images to be analyzed and the height and width of each

numberOfImages = len(allImages)

samplePath = directory + allImages[0]
sampleImage = rawpy.imread(samplePath)

imageHeight = sampleImage.sizes[0]
imageWidth = sampleImage.sizes[1]


imageHeight = 100
imageWidth = 2000

sampleImage.close()

#takes in an images and returns a matrix with the raw pixel values 
#for each pixel of the image

#array for all the pixel counts
print('initializing array')
dataArray = np.empty((imageHeight, imageWidth, numberOfImages))

def writeData(directory, name, dataToWrite):
	with open(directory + name + '.csv', 'w') as f:
		writer = csv.writer(f)
		writer.writerows(dataToWrite)



def getCountMatrix(path, height, width, dataArray, index):
	print('reading', path)
	image = rawpy.imread(path)
	

	# it = np.nditer(image, flags=['multi_index'])
	# while not it.finished:
	# 	countMatrix[it.multi_index] = image.raw_value_visible(it.multi_index)

	for row in range(0, height):
		for column in range(0, width):
			dataArray[row, column, index] = image.raw_value_visible(row, column)
			# if (image.raw_value_visible(row, column) > largest_number):

	#print(largest_number)
	image.close()


#if the number of images to analyze is very large (larger than largeAnalysis), then breaks up the analysis into 
#slices of size sliceSize. It will add countArrays to those 
if(numberOfImages > largeAnalysis):
	sliceSize = 2
	count = 0
	tempCountMatrices = []
	for image in allImages:
		path = directory + image
		tempCountMatrices.append(getCountMatrix(path, imageHeight, imageWidth))
		if len(tempCountMatrices) == sliceSize:
			dataSlice = np.mean(np.dstack(tempCountMatrices), axis=2, dtype=np.float32)
			listCountMatrices.append(dataSlice)
			name = path + 'dataSlice' + str(count)
			writeData(directory, name, dataSlice)
			del tempCountMatrices
			tempCountMatrices = []
			count = count + 1
		print(len(tempCountMatrices)) 
else:
	for image in allImages:
		imNumber = 0
		path = directory + image
		print('reading', path)
		image = rawpy.imread(path)
	

		# it = np.nditer(image, flags=['multi_index'])
		# while not it.finished:
		# 	countMatrix[it.multi_index] = image.raw_value_visible(it.multi_index)

		for row in range(0, imageHeight):
			for column in range(0, imageWidth):
				dataArray[row, column, index] = image.raw_value_visible(row, column)
				# if (image.raw_value_visible(row, column) > largest_number):

		#print(largest_number)
		image.close()
		imNumber = imNumber + 1



# print(stackedCountMatrices.shape)

# countMatrix1 = np.array(
# 	[[1,2,3],
# 	[4,5,6],
# 	[7,8,9]])
# countMatrix2 = np.array(
# 	[[3,2,1],
# 	[6,5,4],
# 	[9,8,7]])
# countMatrix3 = np.array(
# 	[[4,5,6],
# 	[1,2,3],
# 	[7,8,9]])

# stackedCountMatrices = np.dstack([countMatrix1, countMatrix2, countMatrix3])
# print(stackedCountMatrices.shape)
# print(countMatrix3)
# print('full')

print('generating statistics...')
#look back to make sure taking the mean and stdev will be accurate
#after extensive checking, it is accurate and the method used is correct
#dont need 64 bits of precision, 32 reduces memory
meanData = np.ravel(np.mean(stackedCountMatrices, axis=2, dtype=np.float32))

#ddof = 1
#why is this? need to research Keita told me to do
stDevData = np.ravel(np.std(stackedCountMatrices, axis=2, dtype=np.float32, ddof=1))

# print('writing...')

# print(len(meanData))

data = np.empty((len(meanData), 2))

for index in range(0, len(meanData)):
	data[index, 0] = meanData[index]
	data[index, 1] = stDevData[index]


#print(data)

print('writing data...')
writeData(directory, 'dataTest', data)

print('making plots...')
#!!!!the following makes all the plots for data
makePlots(meanData, stDevData)


