'''
Extend Dynamic Range Visualize

Christian Pluchar- SURF17


Starts with slowest shutter speed of an image set, and finds all the pixels which are not saturated. Adds them up, and calculate the power coming from those pixels.
Then, looks at the next slowest shutter speed, finds the not saturated pixels that were not measured in the previous image, and calculates the power from those.
Repeat until there are no saturated pixels left or at the last image. If at the last image, then take all of the remaining pixels and add them up.

Modification to keep all values in their place and then generate image/graph at the end.
'''


import rawpy
import imageio
import numpy as np
from PIL import Image
import glob
import os
import csv

from scatterVis import create3dPlot
from scatterVis2d import create2dPlot


reference_image_path = ''
num_channels = 3


directory = 'ITMx//previousPictures//'
saturated_value = 2**14 - 1
num_channels = 3

#Enter previously found calibration values
#red, green, blue
#in units of joules/count

calibration_values = np.array([1/(1.61e16), (1/(1.45e16)), (1/(1.65E+16))])

#camera calibration doe at ISO 200, linear scaling factors for different ISOs
#ie 1 for ISO 200, 4 for ISO 800
iso_factors = np.array(
	[100/200, 100/200, 100/200, 100/200, 100/200])

#in seconds
#slowest to fastest
exposure_times = np.array([30, 10, 3, 1, 1/3
	])

ambient_background_subtraction = None
dark_offset_subtraction = True

#rgb
#formula from dark offset measurements with D7100
#See August 10th notes
def darkOffsetFunc(exposureTime):
	return  10**(-1 * np.log10(exposureTime) - 8.83)

#power output found by the y-intercept of the ambient noise graph
background_subtraction_value = []


#transmitivity of the lens
lens_transmitivity = .338

#names of folder paths to analyze
#enter in same order as the exposure times
folders = [
'30s',
'10s',
'3s',
'1s',
'Third'
]

#keeps track of where to add the next image in the 2d array of allImages
counter = 0 

#stores where all the images are
allImages = []

#gets all the images organized in the folders and adds them to allImages
#NOTE: will have to change the directory back the location of measureEnergy.py
for folder in folders:
	os.chdir(directory+folder)
	allImages.append([])
	for image in glob.glob('*.NEF'):
		allImages[counter].append(image)
		print(image)
	os.chdir('..')
	os.chdir('..')
	os.chdir('..')
	counter = counter + 1


#main method for analyzing the images
#returns a list with: redMean, greenMean, blueMean, saturated pixels
#for a folder of image names
def getPixelCounts(folder, folderName, saturatedPixels, imageHeight, imageWidth):

	print(folderName, 'contains ', len(folder), 'images in it')

	#keeps track of which pixels have been flipped from saturated to unsaturated
	#simple method for cases with multiple exposures
	#0 for not modified, 1 for modified 
	modified = np.zeros_like(saturatedPixels)

	#keeps track of the number of saturated pixels in each image
	saturatedPixelCounts = np.zeros(len(folder))

	#keeps track of the energy for each pixel for each image
	pixelEnergyMatrix = np.zeros((imageHeight, imageWidth, len(folder)))


	imageCounter = 0
	for path in folder:
		saturated = None
		imagePath = directory + folderName + '//' + path
		readImage = rawpy.imread(imagePath)

		print('Reading ', imagePath)

		#!!!!!!taken from cj's code

		# Colour filter array
		cfa = readImage.raw_colors
		cfa_shape = cfa.shape
		all_colors = readImage.raw_image

		# Create boolean masks for each colour
		red_mask = np.reshape(np.in1d(cfa, 0), cfa_shape)
		green_mask = np.reshape(np.in1d(cfa, (1, 3)), cfa_shape)
		blue_mask = np.reshape(np.in1d(cfa, 2), cfa_shape)


		# Multiplying original array by masks sets all pixels 
		#   of colours other than the desired colour to 0
		red_only = np.multiply(all_colors, red_mask)
		green_only = np.multiply(all_colors,  green_mask)
		blue_only = np.multiply(all_colors, blue_mask)

		#end cj's code


		for height in range(0, imageHeight):
			for width in range(0, imageWidth):

				#simple way address if there are multiple exposures and a pixel is saturated in one and not in the others 
				#just use the determination of the first image analyzed, the others can only "read" saturatedPixels

				#if the pixel was previously found to be saturated/ didn't get counted
				#or it was counted by another exposure
				if saturatedPixels[height, width] == 1 or modified[height, width] == 1:

					if red_only[height, width] == saturated_value or green_only[height,width] == saturated_value or blue_only[height, width] == saturated_value:
						saturated = True
						
					
					else: 
						saturatedPixels[height, width] = 0
						modified[height, width] = 1

						pixelEnergyMatrix[height, width, imageCounter] = (red_only[height, width] * calibration_values[0]) + (green_only[height, width] * calibration_values[1]) + (blue_only[height, width] * calibration_values[2])


		if saturated:
			print(imagePath, ' has overexposed/saturated pixels')

		saturatedPixelCounts[imageCounter] = np.sum(saturatedPixels)
		#print('number of saturated pixels', np.sum(saturatedPixels))

		imageCounter += 1


	meanPixelEnergyMatrix = np.mean(pixelEnergyMatrix, axis=2)

	saturatedPixelMean = np.mean(saturatedPixelCounts)

	numPixModified = np.sum(modified)

	print('exposure sum', np.sum(meanPixelEnergyMatrix))

	return meanPixelEnergyMatrix, saturatedPixels, numPixModified, saturatedPixelMean

#main method
def analyzeImages(allImages, folders):
	

	folderName = folders[0]
	imagePath = directory + folderName + '//' + str(allImages[0][0])
	sampleImage = rawpy.imread(imagePath)
	height = sampleImage.sizes[0]
	width = sampleImage.sizes[1]

	#keeps track of where the saturated pixels are
	#ones are  saturated, zeroes are unsaturated
	#starts out as all saturated so the program looks at all of the pixels on the first image
	saturatedPixels = np.ones((height, width))

	#3d matrix with all mean pixel energies from each folder
	allPixelEnergyMatrix = np.zeros((height, width, len(folders)))

	folderNumber = 0

	modifiedPixCounts = np.zeros(len(folders))
	saturatedPixStats = np.zeros(len(folders))
	for folder in allImages:
		allPixelEnergyMatrix[:, :, folderNumber], saturatedPixels, numPixModified, numSaturatedPixels = getPixelCounts(folder, folders[folderNumber], saturatedPixels, height, width)

		modifiedPixCounts[folderNumber] = numPixModified
		saturatedPixStats[folderNumber] = numSaturatedPixels
		
		folderNumber += 1

	#calulations 
	for index in range(0, exposure_times.size):

		# Power = Energy / Time
		exposureTime = np.full_like(allPixelEnergyMatrix[:, :, index], exposure_times[index])
		allPixelEnergyMatrix[:, :, index] = np.divide(allPixelEnergyMatrix[:, :, index], exposureTime)
		
		#iso scales linearly, this adjusts for that
		#calibration was taken with ISO200
		isoFactor = np.full_like(allPixelEnergyMatrix[:, :, index], iso_factors[index])
		allPixelEnergyMatrix[:, :, index] = np.divide(allPixelEnergyMatrix[:, :, index], isoFactor)

	#Total power scattered = sum of all the powers taken from images

	numOfExposures = exposure_times.size
	darkOffsetStats = np.zeros(numOfExposures)

	#subtracts the electronics noise (dark offset) of the camera based on the proportion 
	#that a perticular exposure time was used
	if dark_offset_subtraction:
		for index in range(0, numOfExposures):
			subtractionPerPixel = ((modifiedPixCounts[index] / (height * width)) * darkOffsetFunc(exposure_times[index])) / (height * width)
			print('number of modified pixels', modifiedPixCounts[index])

			darkOffsetStats[index] = (modifiedPixCounts[index] / (height * width)) * darkOffsetFunc(exposure_times[index])

			darkOffsetSubtractionMatrix = np.full_like(allPixelEnergyMatrix[:, :, index], subtractionPerPixel)
			allPixelEnergyMatrix[:, :, index] = np.subtract(allPixelEnergyMatrix[:, :, index], subtractionPerPixel)

	pixelPowerMatrix  = np.sum(allPixelEnergyMatrix, axis=2)

	#subtracts ambient light in the detector
	if ambient_background_subtraction:
		subtractionPerPixel = background_subtraction_value / (height * width)
		backgroundSubtractionMatrix = np.full_like(pixelPowerMatrix, subtractionPerPixel)

		pixelPowerMatrix = np.subtract(pixelPowerMatrix, backgroundSubtractionMatrix)


	scatteredPower = np.sum(pixelPowerMatrix)
	#lens using to take the images do not have perfect transmitivity 
	scatteredPower = scatteredPower / lens_transmitivity

	lensTransmitivityMatrix = np.full_like(pixelPowerMatrix, lens_transmitivity)
	pixelPowerMatrix = np.divide(pixelPowerMatrix, lensTransmitivityMatrix)

	return pixelPowerMatrix, scatteredPower, darkOffsetStats, saturatedPixStats

pixelPowerMatrix, scatteredPower, darkOffsetStats, saturatedPixStats = analyzeImages(allImages, folders)


print('power', scatteredPower)

np.save('pixelPowerMatrixITMx', pixelPowerMatrix)

print(exposure_times)
print('dark offset contributions:')
print(darkOffsetStats)
print(saturatedPixStats)
hcropSize = 2000
wcropSize = 2500
hcropStart = 1000
wcropStart = 1500
croppedMatrix = np.zeros((hcropSize, wcropSize))

create2dPlot(pixelPowerMatrix)
for height in range(0, hcropSize):
	for width in range(0, wcropSize):
		croppedMatrix[height, width] = pixelPowerMatrix[height + hcropStart, width + wcropStart]

create2dPlot(croppedMatrix)
create3dPlot(pixelPowerMatrix)




